package sun.awt.image;

import java.lang.ref.SoftReference;
import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import sun.awt.AppContext;
import java.awt.Image;
import java.lang.ref.ReferenceQueue;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.LinkedHashMap;

public final class ImageCache
{
    private final LinkedHashMap<ImageCache.PixelsKey, ImageCache.ImageSoftReference> map;
    private final int maxPixelCount;
    private int currentPixelCount;
    private final ReadWriteLock lock;
    private final ReferenceQueue<Image> referenceQueue;
    
    public static ImageCache getInstance() {
        return (ImageCache)AppContext.getSoftReferenceValue(ImageCache.class, ()/*  */ -> new ImageCache());
    }
    
    ImageCache(int maxPixelCount) {
        this.map = new LinkedHashMap(16, 0.75f, true);
        this.currentPixelCount = 0;
        this.lock = new ReentrantReadWriteLock();
        this.referenceQueue = new ReferenceQueue();
        this.maxPixelCount = maxPixelCount;
    }
    
    ImageCache() {
        this(2097152);
    }
    
    public void flush() {
        this.lock.writeLock().lock();
        try {
            this.map.clear();
        }
        finally {
            this.lock.writeLock().unlock();
        }
    }
    
    public Image getImage(ImageCache.PixelsKey key) {
        this.lock.readLock().lock();
        ImageCache.ImageSoftReference imageSoftReference;
        try {
            imageSoftReference = (ImageCache.ImageSoftReference)this.map.get(key);
        }
        finally {
            this.lock.readLock().unlock();
        }
        return (imageSoftReference == null) ? null : ((Image)imageSoftReference.get());
    }
    
    public void setImage(ImageCache.PixelsKey key, Image image) {
        this.lock.writeLock().lock();
        try {
            ImageCache.ImageSoftReference imageSoftReference = (ImageCache.ImageSoftReference)this.map.get(key);
            if (imageSoftReference != null) {
                if (imageSoftReference.get() != null) {
                    return;
                }
                this.currentPixelCount -= key.getPixelCount();
                this.map.remove(key);
            }
            this.currentPixelCount += key.getPixelCount();
            if (this.currentPixelCount > this.maxPixelCount) {
                ImageCache.ImageSoftReference imageSoftReference2;
                while ((imageSoftReference2 = (ImageCache.ImageSoftReference)this.referenceQueue.poll()) != null) {
                    this.map.remove(imageSoftReference2.key);
                    this.currentPixelCount -= imageSoftReference2.key.getPixelCount();
                }
            }
            if (this.currentPixelCount > this.maxPixelCount) {
                Map.Entry<Object, Object> entry;
                for (Iterator iterator = this.map.entrySet().iterator(); this.currentPixelCount > this.maxPixelCount && iterator.hasNext(); this.currentPixelCount -= ((ImageCache.ImageSoftReference)entry.getValue()).key.getPixelCount()) {
                    entry = (Map.Entry)iterator.next();
                    iterator.remove();
                    Image image2 = (Image)((ImageCache.ImageSoftReference)entry.getValue()).get();
                    if (image2 != null) {
                        image2.flush();
                    }
                }
            }
            this.map.put(key, new ImageCache.ImageSoftReference(key, image, this.referenceQueue));
        }
        finally {
            this.lock.writeLock().unlock();
        }
    }
    
    private static class ImageSoftReference extends SoftReference<Image>
    {
        final ImageCache.PixelsKey key;
        
        ImageSoftReference(ImageCache.PixelsKey key, Image referent, ReferenceQueue<? super Image> q) {
            super(referent, q);
            this.key = key;
        }
    }
    
    public interface PixelsKey
    {
        public abstract int getPixelCount();
    }
}
